//
//  SettingPresenter.m
//  AsReaderScan
//
//  Created by Mikiko on 11/7/24.
//  Copyright © 2024 SPS INC. All rights reserved.
//

// SettingPresenter.m
#import "SettingPresenter.h"
#import "Store.h"
#import "Extends.h"
#import "OCRController.h"
#import "SecurityLevelController.h"

@interface SettingPresenter() <AsreaderBarcodeDeviceDelegate, AsReaderDeviceDelegate, OCRDelegate, SecurityLevelControllerDelegate, AutoLaunchControllerDelegate>
{

}
@end

@implementation SettingPresenter
+ (instancetype)sharedInstance
{
    static dispatch_once_t pred = 0;
    __strong static id _sharedObject = nil;
    dispatch_once(&pred, ^{
        _sharedObject = [[SettingPresenter alloc] init];        
    });
    NSLog(@"SettingPresenter:sharedInstance = %@", _sharedObject);
    return _sharedObject;
}

- (void)setDelegate:(id<SettingPresenterDelegate>)delegate {
	if (self) {
		[self setModel:[Store shared]];     
		[[self barcode] setDelegateBarcode:self];
		[[self barcode] setDelegateDevice:self];
		_delegate = delegate;        
	}
}

- (Store *)model {
    return [Store shared];
}

- (AsReaderBarcodeDevice *)barcode {
    return [AsReaderBarcodeDevice sharedInstance];
}

// Symbology methods
- (void)setAllSymbolEnable {
    [[self model] setAllSymbolEnable];
}

- (void)setAllSymbolDisable {
    [[self model] setAllSymbolDisable];
}

- (void)updateSymbology {
    NSLog(@"updateSymbology");
    
    NSMutableDictionary<NSNumber *, BarcodeZebraSsiParam *> *dictParams = [NSMutableDictionary dictionary];
    for (SymbolItemFromArray *item in [self model].symbolTableFromArray) {
        BarcodeZebraSsiParam *aSymbols = [[BarcodeZebraSsiParam alloc] initWithNumber:item.code status:(item.isOn ? 1 : 0) name:item.title];
        dictParams[@(item.code)] = aSymbols;
    }
    
    BOOL isBeepOn = self.model.settingTable[iSwSSIBeep].isOn;
    [self.barcode setSsiParamWithDictionary:dictParams beepOn:isBeepOn];
}

- (void)updateSecurity {
    NSLog(@"updateSecurity");

    NSMutableDictionary<NSNumber *, BarcodeZebraSsiParam *> *dictParams = [NSMutableDictionary dictionary];

    for (SecurityItemFromArray *item in self.model.securityTableFromArray) {
        BarcodeZebraSsiParam *aSecurity = [[BarcodeZebraSsiParam alloc] initWithNumber:item.code
                                                                             status:item.level
                                                                              name:item.title];
        dictParams[@(item.code)] = aSecurity;
    }
    
    [self setSsiParam:dictParams];
}


// Set
- (void)setCodeID:(CodeID)type {
    BOOL isBeepOn = [self getOnOffValue:iSwSSIBeep];
    switch (type) {
        case CodeIDNone:
            [self.barcode setCodeID:CODEID_NONE isBeepOn:isBeepOn];
            break;
        case CodeIDAim:
            [self.barcode setCodeID:CODEID_AIM isBeepOn:isBeepOn];
            break;
        case CodeIDSymbol:
            [self.barcode setCodeID:CODEID_SYMBOL isBeepOn:isBeepOn];
            break;
    }
}

- (BOOL)getOnOffValue:(IDs)index {
    return self.model.settingTable[index].isOn;
}

- (void)setSleepTime:(NSInteger)time {
    [self.barcode setSleepTime:(int32_t)time];
}

- (void)setEnableAndroidHID:(BOOL)isOn {
    NSLog(@"setEnableAndroidHID %d", isOn);
    [self.barcode setAndroidHIDEnable:isOn];
}

- (void)setEnableiOSHID:(BOOL)isOn {
    NSLog(@"setEnableiOSHID %d", isOn);
    [self.barcode setiOSHIDEnable:isOn];
}

- (void)setPresentationMode:(BOOL)isOn {
    BOOL isBeepOn = [self getOnOffValue:iSwSSIBeep];
    [self.barcode setPresentationMode:isOn isBeepOn:isBeepOn];
}

- (void)setSleepBeep:(BOOL)isOn {
    [self.barcode setSleepBeep:isOn];
}

- (void)setOCRType:(TypeOCR)type {
    BOOL isBeepOn = [self getOnOffValue:iSwSSIBeep];
    
    switch (type) {
        case OCRNone:
            [self.barcode setOCRType:OCR_TYPENONE isBeepOn:isBeepOn];
            break;
        
        case OCRA:
            [self.barcode setOCRType:OCR_TYPEA isBeepOn:isBeepOn];
            break;
        
        case OCRB:
            [self.barcode setOCRType:OCR_TYPEB isBeepOn:isBeepOn];
            break;
        
        case OCRAB:
            [self.barcode setOCRType:OCR_TYPEAB isBeepOn:isBeepOn];
            break;
    }
}

- (void)setSsiParam:(NSDictionary<NSNumber *, BarcodeZebraSsiParam *> *)dict {
    BOOL isBeepOn = [self getOnOffValue:iSwSSIBeep];
    [self.barcode setSsiParamWithDictionary:dict beepOn:isBeepOn];
}

- (void)setAutoLaunch:(BOOL)isOn bundle:(NSString *)bundle {
    [self.barcode setAutoLaunch:isOn bundleId:(NSString *)bundle];
}





//received
- (void)receivedCodeID:(int32_t)codeID {
    NSString *selectItem = @"";
    NSInteger selectIndex = (NSInteger)codeID;

    NSLog(@"CodeID %ld", (long)selectIndex);
    NSArray *codeArray = [ReaderInfo codeIDArray];
    
    if (selectIndex < codeArray.count) {
        selectItem = codeArray[selectIndex];
        switch (selectIndex) {
            case 0:
                self.model.codeIDType = CodeIDNone;
                break;
            case 1:
                self.model.codeIDType = CodeIDAim;
                break;
            case 2:
                self.model.codeIDType = CodeIDSymbol;
                break;
        }
        NSLog(@"CodeID %ld %@", (long)selectIndex, selectItem);
    }
    
    [self settingReceivedCodeID:selectItem];
}

- (void)receivedSleepTime:(int32_t)time {
    NSLog(@"Sleep Time");
    NSLog(@"여기!!!!!! 5");
     
    NSString *selectItem = @"";
    NSInteger selectIndex = (NSInteger)time;

    NSLog(@"Time %ld", (long)selectIndex);

    NSString *compareString = [ReaderInfo sleepTimeStringFromIndex:selectIndex];
    for (NSString *select in [ReaderInfo sleepTimeArray]) {
        if ([compareString isEqualToString:select]) {
            selectItem = select;
            NSLog(@"Time: %@", selectItem);
        }
    }

    NSLog(@"Step1 - SleepTime");
    [self settingReceivedSleepTime:selectItem];
}

- (void)plugged:(BOOL)plug { 
    
}


- (void)readerConnected:(int)status { 
    
}


- (void)receivedHID:(BOOL)hidOn receivediOSHID:(BOOL)iOShidOn {
    if ([self respondsToSelector:@selector(receivedHID:receivediOSHID:)]) {
        [self settingReceivedHID:hidOn receivediOSHID:iOShidOn];
    }
}

- (void)receivedPresentationMode:(BOOL)isOn {
    if ([self respondsToSelector:@selector(receivedPresentationMode:)]) {
        [self settingReceivedPresentationMode:isOn];
    }
}

- (void)receivedSleepBeep:(NSData *)data {
    if ([self.delegate respondsToSelector:@selector(receivedSleepBeep:)]) {
        [self settingReceivedSleepBeep:data];
    }
}

- (void)receivedOCR:(BOOL)isNone ocrAon:(BOOL)isOCRAon ocrBOn:(BOOL)isOCRBon {
    NSLog(@"Reader Mngr: OCR %@ %@ %@", isNone ? @"YES" : @"NO", isOCRAon ? @"YES" : @"NO", isOCRBon ? @"YES" : @"NO");
    
    NSInteger selectItem = 0;
    
    // 0 : None, 1: OCRA, 2: OCRB, 3: OCR AB
    if (isNone) {
        selectItem = 0;
    }
    
    if (isOCRAon) {
        selectItem = 1;
    }
    
    if (isOCRBon) {
        selectItem = 2;
    }
    
    if (isOCRAon && isOCRBon) {
        selectItem = 3;
    }
    
    [self settingReceivedOCR:selectItem];
}

- (void)receivedBarcodeSecurity:(NSDictionary<NSString *, id> *)security {
    NSInteger level = 0;
    
    for (id key in security) {
        id value = security[key];
        
        if ([value isKindOfClass:[BarcodeZebraSsiParam class]]) {
            BarcodeZebraSsiParam *param = (BarcodeZebraSsiParam *)value;
            level = param.status;
            [self.model setSecurityLevel:param.name level:level];
        }
    }
    
    [self settingReceivedBarcodeSecurity];
    NSLog(@"receivedBarcodeSecurity -> model.setSecurityLevel");
}

- (void)receivedGetAutoLaunch:(BOOL)isOn bundle:(NSString *)data {
    NSLog(@"receivedGetAutoLaunch: %@ %d", data, isOn);
    [self settingReceivedGetAutoLaunch:isOn bundle:data];
}

//MARK: -- API Setting 화면
- (void)settingReceivedCodeID:(NSString *)value {
    UIViewController *vc = self.viewController;
    if (!vc) return;
    
    [UIAlertController showSelectAlert:vc
                                 title:@"Code ID"
                               message:@"Select Code ID type"
                                 lists:[ReaderInfo codeIDArray]
                           cancelTitle:@"Cancel"
                              itemName:value
                      selectionHandler:^(NSInteger selectedIndex) {
        NSLog(@"select index %ld", (long)selectedIndex);
        CodeID type = [ReaderInfo codeIDFromIndex:selectedIndex];
        if (type >= 0) {
            [self setCodeID:type];
        }
    }];
}

- (void)settingReceivedSleepTime:(NSString *)value {
    UIViewController *vc = self.viewController;
    if (!vc) return;
    
    [UIAlertController showSelectAlert:vc
                                 title:@"Sleep Time"
                               message:@"Select Sleep Time"
                                 lists:[ReaderInfo sleepTimeArray]
                           cancelTitle:@"Cancel"
                              itemName:value
                      selectionHandler:^(NSInteger selectedIndex) {
        NSLog(@"select index %ld", (long)selectedIndex);
        NSInteger type = [ReaderInfo sleepTimeFromIndex:selectedIndex];
        if (type >= 0) {
            [self setSleepTime:type];
        }
    }];
}

- (void)settingReceivedHID:(BOOL)hidOn receivediOSHID:(BOOL)iOShidOn {
    NSLog(@"Received HID value: %@", hidOn ? @"YES" : @"NO");
    NSLog(@"Received iOShidOn value: %@", iOShidOn ? @"YES" : @"NO");
    
    if (hidOn) {
        self.model.settingTable[iSwAndroidHID].isOn = YES;
        self.model.settingTable[iSwiOSHID].isOn = NO;
        [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"iSwAndroidHID"];
        [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"iSwiOSHID"];
    } else if (iOShidOn) {
        self.model.settingTable[iSwAndroidHID].isOn = NO;
        self.model.settingTable[iSwiOSHID].isOn = YES;
        [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"iSwAndroidHID"];
        [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"iSwiOSHID"];
    } else {
        self.model.settingTable[iSwAndroidHID].isOn = NO;
        self.model.settingTable[iSwiOSHID].isOn = NO;
    }
    
    [self.delegate updatedReaderInfo];
}

- (void)settingReceivedPresentationMode:(BOOL)isOn {
    NSLog(@"Received Presentation mode: %@", isOn ? @"YES" : @"NO");
    self.model.settingTable[iSwPM].isOn = isOn;
    [self.delegate updatedReaderInfo];
}

- (void)settingReceivedSleepBeep:(NSData *)data {
    NSLog(@"Received SleepBeep data: %@", data);
}

- (void)settingReceivedOCR:(NSInteger)value {
    NSArray<NSString *> *select = @[];
    
    switch (value) {
        case 0:
            NSLog(@"OCR NONE");
            break;
        case 1:
            select = @[@"OCRA"];
            NSLog(@"OCR A");
            break;
        case 2:
            select = @[@"OCRB"];
            NSLog(@"OCR B");
            break;
        case 3:
            select = @[@"OCRA", @"OCRB"];
            NSLog(@"OCR AB");
            break;
        default:
            NSLog(@"default");
            break;
    }
    
    UIViewController *vc = self.viewController;
    if (!vc) return;
    
    OCRController *ocrVC = [[OCRController alloc] init];
    ocrVC.delegate = self;
    ocrVC.modalPresentationStyle = UIModalPresentationOverCurrentContext;
    [ocrVC setDataWithList:@[@"OCRA", @"OCRB"] selectList:select];
    
    [vc presentViewController:ocrVC animated:YES completion:nil];
}

- (void)didSelectOptions:(NSSet<NSString *> *)options {
    NSLog(@"List selected %@", options);
    
    if ([options containsObject:@"OCRA"] && [options containsObject:@"OCRB"]) {
        NSLog(@"OCR Select : AB");
        [self setOCRType:OCRAB];
        return;
    }
    
    if ([options containsObject:@"OCRA"]) {
        [self setOCRType:OCRA];
        NSLog(@"OCR Select : A");
    } else if ([options containsObject:@"OCRB"]) {
        [self setOCRType:OCRB];
        NSLog(@"OCR Select : B");
    } else {
        [self setOCRType:OCRNone];
        NSLog(@"OCR Select : NONE");
    }
}

- (void)settingReceivedBarcodeSecurity {
    UIViewController *vc = self.viewController;
    if (!vc) return;
    
    SecurityLevelController *securityLevelVC = [[SecurityLevelController alloc] init];
    securityLevelVC.delegate = self;
    securityLevelVC.modalPresentationStyle = UIModalPresentationOverCurrentContext;
    [vc presentViewController:securityLevelVC animated:YES completion:nil];
}

- (void)settingReceivedGetAutoLaunch:(BOOL)isOn bundle:(NSString *)bundle {
    NSLog(@"settingReceivedGetAutoLaunch = %@", self.viewController);
    UIViewController *vc = self.viewController;
    if (!vc) {
        return;
    }
    
    // 이미 모달이 표시되고 있는지 확인
    if (self.autoLaunchController.presentingViewController != nil) {
        NSLog(@"AutoLaunchController is already presented.");
        return;
    }
    
    // 새로운 모달 인스턴스를 생성
    if (!self.autoLaunchController) {
        self.autoLaunchController = [[AutoLaunchController alloc] init];
        self.autoLaunchController.delegate = self;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        self.autoLaunchController.readerCurrentBundleIdLabel.text = bundle;
        [self.autoLaunchController.swAction setOn:isOn animated:YES];
        self.autoLaunchController.readerBundleIdTextField.text = @"";
    });
    
    // 모달 표시
    self.autoLaunchController.modalPresentationStyle = UIModalPresentationFullScreen;
    [vc presentViewController:self.autoLaunchController animated:YES completion:nil];
}

- (void)settingReceivedSetAutoLaunch:(BOOL)isOn bundle:(NSString *)bundle {
}

- (void)settingReceivedAutoLaunch:(NSData *)data {
    NSLog(@"Received AutoLaunch: %@", data);
    
    NSString *stringValue = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    if (stringValue) {
        NSLog(@"Received data as string: %@", stringValue);
    } else {
        NSLog(@"Failed to convert data to string");
    }
}



- (void)receivedSymbologies:(NSDictionary*)symbols {
    Store *model = [Store shared];
    
    for (id key in symbols) {
        BarcodeZebraSsiParam *sym = symbols[key];
        //NSLog(@"[%@] %ld,%ld,%@",key,  (long)sym.number,(long)sym.status, sym.name);
        BOOL enable = (sym.status == 0) ? NO : YES;
        [model setSymbolEnable:sym.name isEnable:enable];        
    }
    
    NSLog(@"receivedSymbologies -> model.setSymbolEnable");
}



// API Setting methods
- (void)getHIDMode {
    [[self barcode] getHID];
}

- (void)getPresentationMode {
    [[self barcode] getPresentationMode];
}

- (void)getSymbology {
    [[self barcode] getSymbologies];
}

- (void)getSecurity {
    [[self barcode] getSecurity];
}

- (void)getOCR {
    [[self barcode] getOCR];
}

- (void)getAutoLaunch {
    [[self barcode] getAutoLaunch];
}

- (void)settingAutoLaunch:(BOOL)isOn bundle:(NSString *)bundle {
    [self setAutoLaunch:isOn bundle:bundle];
}

//MARK: Barocde 설정 Switch 이벤트 처리 함수
- (void)changeSwitch:(BOOL)isOn tagID:(NSInteger)tagID {
    NSLog(@"Changing switch: %@ for tagID: %ld", isOn ? @"ON" : @"OFF", (long)tagID);
    for (NSInteger index = 0; index < self.model.settingTable.count; index++) {
        NSInteger ids = [self.model.settingTable[index] ids];
        if (ids == tagID) {
            [self.model toggleSettingSwitch:self.model.settingTable[index]];
            NSLog(@"New state for ID %ld: %@", (long)ids, self.model.settingTable[index].isOn ? @"ON" : @"OFF");
            break;
        }
    }
    
    [self.delegate updatedReaderInfo];
    
    IDs ids = [self getEnumFromNumber:tagID];
    switch (ids) {
        case iSwCR:
            NSLog(@"proper APP 저장");
            break;
        case iSwTM:
            NSLog(@"proper APP 저장");
            break;
        case iSwBep:
        case iSwVib:
        case iSwLED:
        case iSwAim:
        case iSwPbp:
        {
            [self.barcode setBeep:self.model.settingTable[iSwBep].isOn vibration:self.model.settingTable[iSwVib].isOn batteryGaugeLed:self.model.settingTable[iSwLED].isOn barcodeAimer:self.model.settingTable[iSwAim].isOn barcodePowerOnBeep:self.model.settingTable[iSwPbp].isOn];
            break;
        }
        case iSwSleepBeep:
            [self setSleepBeep:isOn];
            break;
        case iSwSSIBeep:
            self.model.readerInfo.ssiBeep = isOn;
            break;
        case iSwAndroidHID:
        case iSwiOSHID:
            [self handleHIDSettings:isOn receivediOSHID:isOn forIDs:ids];
            break;
        case iSwPM:
            [self setPresentationMode:isOn];
            break;
        default:
            NSLog(@"");
            break;
    }
}

//MARK:  Barocde 설정 Button Action 이벤트 처리 함수
- (void)handleHIDSettings:(BOOL)hidOn receivediOSHID:(BOOL)iOShidOn forIDs:(IDs)ids {
    if (ids == iSwAndroidHID) {
        if (hidOn) {
            self.model.settingTable[iSwAndroidHID].isOn = YES;
            self.model.settingTable[iSwiOSHID].isOn = NO;
            [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"iSwAndroidHID"];
            [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"iSwiOSHID"];
        } else {
            self.model.settingTable[iSwAndroidHID].isOn = NO;
            [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"iSwAndroidHID"];
        }
        NSLog(@"Set iSwAndroidHID to %@", self.model.settingTable[iSwAndroidHID].isOn ? @"YES" : @"NO");
        NSLog(@"Set iSwiOSHID to %@", self.model.settingTable[iSwiOSHID].isOn ? @"YES" : @"NO");
        [self setEnableAndroidHID:hidOn];
    } else if (ids == iSwiOSHID) {
        if (iOShidOn) {
            self.model.settingTable[iSwAndroidHID].isOn = NO;
            self.model.settingTable[iSwiOSHID].isOn = YES;
            [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"iSwAndroidHID"];
            [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"iSwiOSHID"];
            [self setEnableiOSHID:YES];
        } else {
            self.model.settingTable[iSwiOSHID].isOn = NO;
            [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"iSwiOSHID"];
            [self setEnableiOSHID:NO];
        }
        NSLog(@"Set iSwiOSHID to %@", self.model.settingTable[iSwiOSHID].isOn ? @"YES" : @"NO");
        NSLog(@"Set iSwAndroidHID to %@", self.model.settingTable[iSwAndroidHID].isOn ? @"YES" : @"NO");
    }
    [self.delegate updatedReaderInfo];
}


- (void)btnEvent:(NSInteger)tagID viewController:(UIViewController *)vc {
    self.viewController = vc;
    
    NSLog(@"btnEvent = %@", vc);
    
    IDs ids = [self getEnumFromNumber:tagID];
    if (ids == -1) { // Assuming -1 is not a valid value in your enum
        return;
    }
    
    NSLog(@"TAG ID %ld", (long)ids);
    
    switch (ids) {
        case iBtOCR:
            [self.barcode getOCR];
            NSLog(@"iBtOCR");
            break;
        case iBtSleeptime:
            [self.barcode getSleepTime];
            NSLog(@"iBtSleeptime");
            break;
        case iBtnCodID:
            [self.barcode getCodeID];
            NSLog(@"iBtnCodID");
            break;
        case iBtSecLevel:
            [self.barcode getSecurity];
            NSLog(@"iBtSecLevel");
            break;
        case iBtAutoLaunch:
            [self.barcode getAutoLaunch];
            NSLog(@"iBtAutoLaunch");
            break;
        default:
            break;
    }
}

- (IDs)getEnumFromNumber:(NSInteger)number {
    return (IDs)number;
}


- (BOOL)getOnOff:(IDs)tagID {
    for (SettingItem *item in self.model.settingTable) {
        if (item.ids == tagID) {
            return item.isOn;
        }
    }
    return NO;
}

- (void)selectCodeID:(UIViewController *)viewController {
}

- (void)selectSleepTime:(UIViewController *)viewController {
}

- (void)selectOCRUse:(UIViewController *)viewController {
}


//
//- (void)getAutoLaunch {
//}
//
//- (void)getPresentationMode {
//}
//
//- (void)getSecurity {
//}
//
//- (void)getSymbology {
//}
//
//- (void)setAllSymbolDisable {
//}
//
//- (void)setAllSymbolEnable {
//}
//
//- (void)setSymbolEnable:(NSString *)name isEnable:(BOOL)isEnable {
//}
//
//- (void)updateSecurity {
//}
//
//- (void)updateSymbology {
//}
//
//- (void)getHIDMode {
//}


- (void)setSymbolEnable:(NSString *)name isEnable:(BOOL)isEnable {
}

@end

